/*********************************************************
**********                                      **********
**********  PROGRAM: PLUTO ROBOTIC CONTROL UNIT **********
**********           Version W for WALL-E       **********
**********  Fw version: 1.00                    **********
**********  BC.SOLARSKI (C) 2008/10             **********
**********  solarskit.wz.cz                     **********
**********  solarskit@gmail.com                 **********
**********  MC9S12DP512                         **********
**********  BUSclk = 1.8432 MHz                 **********
**********	     T = 542.53 ns                  **********
**********	last revision:                      **********
**********	30.10.2008 2005                     **********
**********                                      **********
*********************************************************/

/******************************************************************************
	Addition Required Headers/Libraries
******************************************************************************/

// common defines and macros
#include <hidef.h>
// derivative information
#include <mc9s12dp512.h>
//
#pragma LINK_INFO DERIVATIVE "mc9s12dp512b"

/******************************************************************************
	Include
******************************************************************************/

#include "define.c"
#include "init.c"
#include "sci.c"
#include "rs485.c"

/******************************************************************************
	Variables Definition And Declaration
******************************************************************************/

// ***** time counters *****
unsigned int  TimerRun;
unsigned int  TimerAccelerate;

// ***** analog *****
float Volt, VoltBat1, VoltBat2, VoltBat3, VoltBat4;
float VoltX, VoltY, VoltZ;
float CurrentL, CurrentR;

// ***** data ******
// data array
byte data[ 12 ];
// array converted into a pointer
static byte *data_p = &data[ 0 ];

/******************************************************************************
	MAIN Function
******************************************************************************/

void main(void)
{

  EnableInterrupts;
  
  HCS12init(); 
  
  RS485Init();
 
 
/******************************************************************************
	INFINITE LOOP
******************************************************************************/

  for(;;)
  {
	  // analog
	  VoltBat1 = ( BAT1 * ( 1 / BAT1koef ) * VREF ) / 1024.0F;
	  VoltBat2 = ( ( BAT2 * ( 1 / BAT2koef ) * VREF ) / 1024.0F ) - VoltBat1;
	  VoltBat3 = ( ( BAT3 * ( 1 / BAT3koef ) * VREF ) / 1024.0F ) - VoltBat2 - VoltBat1;
	  VoltBat4 = ( ( BAT4 * ( 1 / BAT4koef ) * VREF ) / 1024.0F ) - VoltBat3 - VoltBat2 - VoltBat1;
	  Volt     = ( BAT4 * ( 1 / BAT4koef ) * VREF ) / 1024.0F;
	  
	  CurrentL = ( ( CURRENT_LEFT * VREF * 375.0F ) / 100.0F ) / 1024.0F;
	  CurrentR = ( ( CURRENT_RIGHT * VREF * 375.0F ) / 100.0F ) / 1024.0F;
	  
	  VoltX = ( XOUT * VREF ) / 1024.0F;
	  VoltY = ( YOUT * VREF ) / 1024.0F;
	  VoltZ = ( ZOUT * VREF ) / 1024.0F;
/*	  
	  // undervoltage detection
	  if ( ( VoltBat1 < 3.3F ) ||
	       ( VoltBat2 < 3.3F ) ||
	       ( VoltBat3 < 3.3F ) ||
	       ( VoltBat4 < 3.3F )
	     )
	  {
	    PWR = 1;
	    ERR = 1;
	    
	    ENABLE_LEFT = 0;
	    ENABLE_RIGHT = 0;
	  }
*/
  }
}

/******************************************************************************
	INTERRUPTS ROUTINES
******************************************************************************/

// ***** TIMER 0 IRQ *****
// Events timer 1ms
void interrupt 8 TIMER0( void )
{
  // next interrupt secure
  TC0 += TIMER_1MS;    
  // interuupt flag deactivate
  TFLG1_C0F = 1;
  
  // hart beat timer  
  if ( ++TimerRun >= TIME_RUN / 2 )
  {
    TimerRun = 0;     
    RUN ^= 1;
  }
  
  // lost connection
  if ( TimerLostConnect > 0 )
    TimerLostConnect--;
  else
  {
    PWM1L = 0;
    PWM2L = 0;
    PWM1R = 0;  
    PWM2R = 0;
    CMD = 0;
  }
  
  // acceleration
  if ( ++TimerAccelerate >= TIME_ACCELERATE )
  {
    TimerAccelerate = 0;
  
    if ( PWM1L != PWM1Ldemand )
    {
      if ( PWM1Ldemand > PWM1L )                 
      {
        if ( PWM2L < 5 )
          PWM1L++;
      }
      else
      {
        PWM1L--;       
      }
    }
    
    if ( PWM2L != PWM2Ldemand )
    {
      if ( PWM2Ldemand > PWM2L )                 
      {
        if ( PWM1L < 5 )
          PWM2L++;
      }
      else
      {
        PWM2L--;
      }
    }
    
    if ( PWM1R != PWM1Rdemand )
    {
      if ( PWM1Rdemand > PWM1R )                 
      {
        if ( PWM2R < 5 )
          PWM1R++;
      }
      else
      {
        PWM1R--;
      }
    }
    
    if ( PWM2R != PWM2Rdemand )
    {
      if ( PWM2Rdemand > PWM2R )                 
      {
        if ( PWM1R < 5 )
          PWM2R++;
      }
      else
      {
        PWM2R--;
      }
    }
    
  }
}
