/**************************************************
**********                               **********
**********  PROGRAM: RS485 FUNCTIONS     **********
**********  for PLUTO D ROBOTIC UNIT     ********** 
**********  used module SCI1             **********
**********  Version: 1.00                **********
**********  BC.SOLARSKI (C) 2008/09      **********
**********  solarskit.wz.cz              **********
**********  solarskit@gmail.com          **********
**********  MC9S12DP512                  **********
**********  BUSclk = 25.0000 MHz         **********
**********	     T = 40 ns               **********
**********	last revision:               **********
**********	30.08.2008 1342              **********
**********                               **********
**************************************************/

/******************************************************************************
	B Y T E   D A T A   F O R M A T
	
	1 - adress
	2 - adrees
	3 - N
	
	data[ N ]
	
	last - CRC8ATM	
	
******************************************************************************/

/******************************************************************************
	I N C L U D E
******************************************************************************/

// CRC 8 bit ATM
#include "crc8.c"

/******************************************************************************
	D E F I N I T I O N S
******************************************************************************/

#define RS485TRANSMIT         PTS_PTS4
#define RS485TRANSMIT_DDR     DDRS_DDRS4

#define RS485_BAUD            9600U
#define MODULE_CLK            25000000U

#define RS485_BUFFER_SIZE     32U

#define RS485_DATA_COUNT      16U

#define DATA_VOID             0U
#define DATA_OK               1U
#define DATA_LOST             2U
#define DATA_INVALID          3U

/******************************************************************************
  V A R I A B L E S   D E C L A R A T I O N
******************************************************************************/

// input buffer
byte BufferRS485[ RS485_BUFFER_SIZE ];
// pointer in buffer
unsigned int BufferRS485Ct;
// buffer pointer - static
static byte * ReceivedDataRS485 = &BufferRS485[ 0 ];
// only to clear flag - flag is cleared by read
unsigned char DummyReadRS485;
// data packet to transmit with header and crc
byte RS485AllData[ RS485_DATA_COUNT ];
// array converted into a pointer
static byte * RS485Data_p = &RS485AllData[ 0 ];
// status reg 0 - void, 1 - valid data, 2 - data lost, 3 - invalid data
byte RS485ReceiveStatus;

/******************************************************************************
  F U N C T I O N S
******************************************************************************/

// Initialization
void RS485Init ( void )
{
  // baud
  SCI1BD = MODULE_CLK / RS485_BAUD / 16;
  // receiver interrupt enbale
  SCI1CR2_RIE = 1;
  // receiver enable
  SCI1CR2_RE = 1;
  // transmitter enable
  SCI1CR2_TE = 1; 
  
  // direction pin - out
  RS485TRANSMIT_DDR = 1;
  // 0 - receive, 1 - transmit
  RS485TRANSMIT = 0;
}

// send byte
void RS485SendByte ( byte Data )
{
  // wait for output buffer empty
  while ( !( SCI1SR1 & 0x80 ) );
  SCI1DRL = Data;
}

// Send data
// 1Adr 2Adr 3N 4data0 5data1 6data2 .... dataN-1 CRC
void RS485SendData ( byte Adr1, byte Adr2 , byte * data , byte count )
{
  int j;
  
  RS485AllData[ 0 ] = Adr1;
  RS485AllData[ 1 ] = Adr2;
  RS485AllData[ 2 ] = count;
  
  for ( j = 0 ; j < count ; j++ )
    RS485AllData[ 3 + j ] = *data++;
  
  RS485AllData[ count + 3 ] = CalcCRC8ATM( RS485Data_p , count + 3 );
  
  RS485TRANSMIT = 1;
  
  for ( j = 0 ; j < ( count + 6 ) ; j++ )
    RS485SendByte( RS485AllData[ j ] );
    
  RS485TRANSMIT = 0;  
}

void RS485ClearBuffer ( void )
{
  int j;
  
  for ( j = 0 ; j < RS485_BUFFER_SIZE ; j++ )
    BufferRS485[ j ] = 0x00;
    
  BufferRS485Ct = 0;
}

/******************************************************************************
  I N T E R R U P T
******************************************************************************/
// interrupt is generated when byte was received
void interrupt 21 RS485( void )
{
  // clear flag
  DummyReadRS485 = SCI1SR1;
  // received byte move to buffer
  BufferRS485[ BufferRS485Ct ] = SCI1DRL;
  // buffer overflow protection
  if ( ++BufferRS485Ct >= RS485_BUFFER_SIZE )
    BufferRS485Ct = 0;
  
  if ( RS485ReceiveStatus != 0 )
    RS485ReceiveStatus = DATA_LOST;
    
  // all message received
  if ( 
       ( BufferRS485[ 2 ] != 0 ) &&
       ( BufferRS485Ct == ( BufferRS485[ 2 ] + 4 ) )
     )
  {
    if ( 
         BufferRS485[ BufferRS485[ 2 ] + 3 ] == CalcCRC8ATM( ReceivedDataRS485 , BufferRS485[ 2 ] + 3 )
       )
      RS485ReceiveStatus = DATA_OK;
    else
      RS485ReceiveStatus = DATA_INVALID;
  }
}



