/**************************************************
**********                               **********
**********  PROGRAM: SCI FUNCTIONS       **********
**********  SCI0 for BT commucation      **********
**********  Version: 1.00                **********
**********  BC.SOLARSKI (C) 2008/10      **********
**********  solarskit.wz.cz              **********
**********  solarskit@gmail.com          **********
**********  MC9S12DP512                  **********
**********  XTAL =  3.6864 MHzv          **********
**********  BUSclk = 1.8432 MHz          **********
**********	     T = 542.53 ns           **********
**********	last revision:               **********
**********	30.10.2008 1928              **********
**********                               **********
**************************************************/

/******************************************************************************
	Addition Required Headers/Libraries
******************************************************************************/

// strings
#include <string.h>
// funkce itoa()
#include <STDLIB.h>

/******************************************************************************
	Variables Definition And Declaration
******************************************************************************/

// receiver buffer
unsigned short BufferSCI0[ SCI0_BUFFER_SIZE ];
// counter in buffer
unsigned int BufferCt0;
// read state register
unsigned short DummyRead0;
unsigned int  TimerLostConnect;

// to avoid noise two comparable vaule have to be received
int Byte1store0, Byte2store0;
int Byte1store1, Byte2store1;

int PWM1Ldemand, PWM2Ldemand;
int PWM1Rdemand, PWM2Rdemand;


/******************************************************************************
	SCI0 INTERRUPT HANDLER
******************************************************************************/

// generate on each char is received
void interrupt 20 SCI0( void )
{
  // to clear flag
  DummyRead0 = SCI0SR1;
  // received data are placed into buffer 
  BufferSCI0[ BufferCt0 ] = SCI0DRL; 
  
  // end mark detection
	if (BufferSCI0[ BufferCt0 ] == 205 )
	{
	  // lost connection timer
	  TimerLostConnect = TIME_LOST_CONNECT;
	  CMD = 1;
	  
	  // motor right
	  Byte1store1 = BufferSCI0[ BufferCt0 - 1 ];
	  if ( ( Byte1store1 <= 200 ) && ( Byte1store1 >= 0 ) &&
	       ( Byte1store1 > ( Byte1store0 - 5 ) ) &&
	       ( Byte1store1 < ( Byte1store0 + 5 ) ) 
	      )
	  {
	    if ( Byte1store1 > 105 )
	    // right forward
	    {
	      PWM1Rdemand = 0;
	      PWM2Rdemand = Byte1store1 - 100;
	    }
	    else if ( Byte1store1 < 95 )
	    {
	    // right reverse
	      PWM2Rdemand = 0;
	      PWM1Rdemand = 100 - Byte1store1;
	    }
	    else
	    {
	      PWM1Rdemand = 0;
	      PWM2Rdemand = 0;
	    }
	  }
	  Byte1store0 = Byte1store1;
	  
	  // motor left
	  Byte2store1 = BufferSCI0[ BufferCt0 - 2 ];
	  if ( ( Byte2store1 <= 200 ) && ( Byte2store1 >= 0 ) &&
	       ( Byte2store1 > ( Byte2store0 - 5 ) ) &&
	       ( Byte2store1 < ( Byte2store0 + 5 ) ) 
	      )
	  {
	    if ( Byte2store1 > 105 )
	    // left forward
	    {
	      PWM1Ldemand = 0;
	      PWM2Ldemand = Byte2store1 - 100;
	    }
	    else if ( Byte2store1 < 95 )
	    {
	    // left reverse
	      PWM2Ldemand = 0;
	      PWM1Ldemand = 100 - Byte2store1;
	    }
	    else
	    {
	      PWM1Ldemand = 0;
	      PWM2Ldemand = 0;
	    }
	  }
	  Byte2store0 = Byte2store1;
	  
	  // clear buffer
	  BufferCt0 = 0;
	}
	   
  // buffer overflowing
  if ( ++BufferCt0 >= SCI0_BUFFER_SIZE )
    BufferCt0 = 0;
}